@@ -43,6 +43,19 @@ gboolean tv_focus_out (GtkWidget *widget,
 void on_unknown_year_toggled (GtkToggleButton *togglebutton, gpointer user_data);
 void on_name_clicked (GtkButton *button, gpointer user_data);
 
+
+
+/* this is the filter for phone number edits */
+static gboolean
+phone_key_press_event (GtkWidget *widget, GdkEventKey *k, gpointer p)
+{
+  if (!strstr(" +0123456789-/()",k->string)) 
+    {
+      return TRUE;
+    }
+  return FALSE;
+}
+
 static void
 add_tag (gchar *tag, GtkWidget *w, GtkWidget *pw)
 {
@@ -71,6 +84,13 @@ pop_singles (GtkWidget *vbox, GSList *list, GtkWidget *pw, gboolean visible)
           GtkWidget *l;
           
           add_tag (e->tag, w, pw);
+          if (strstr(e->tag,".TELEPHONE")
+              || strstr(e->tag,".MOBILE") 
+              || strstr(e->tag,".FAX"))
+            {
+               g_signal_connect (G_OBJECT (w), "key_press_event", 
+		         G_CALLBACK (phone_key_press_event), NULL);
+            }
           if (strcmp(e->tag,"NAME")) /* the name field on a button */
             {
               l = gtk_label_new (e->name);
@@ -82,6 +102,7 @@ pop_singles (GtkWidget *vbox, GSList *list, GtkWidget *pw, gboolean visible)
               g_signal_connect(G_OBJECT(l),"clicked",
                                G_CALLBACK(on_name_clicked),
                                gtk_widget_get_toplevel(pw));
+              GTK_WIDGET_SET_FLAGS(w,GTK_CAN_DEFAULT);
             }
           gtk_table_attach (GTK_TABLE (table),
                     l,
@@ -542,11 +563,21 @@ edit_person (struct person *p)
           gchar *tag = gtk_object_get_data (GTK_OBJECT (w), "db-tag");
           struct tag_value *v = db_find_tag (p, tag);
           guint pos = 0;
+          if (tag)
+            {
+               if (!strcasecmp(tag,"NAME"))
+                 {
+                   gtk_widget_grab_default(w);
+                   gtk_widget_grab_focus(w);
+                 }
+            }
           if (v && v->value)
             {
               if (GTK_IS_EDITABLE (w))
-                gtk_editable_insert_text (GTK_EDITABLE (w), v->value,
+                {
+                  gtk_editable_insert_text (GTK_EDITABLE (w), v->value,
                           strlen (v->value), &pos);
+                }
               else if (GTK_IS_DATE_COMBO(w))
                 {
                   if (v->value)
@@ -683,7 +714,7 @@ update_edit (struct person *p, GtkWidget *w)
           ts = n1;
         }
       for (i=0;i<namenum;i++)
-        gtk_entry_set_text(GTK_ENTRY(nameentry[i]),ts);
+        gtk_entry_set_text(GTK_ENTRY(nameentry[i]),g_strstrip(ts));
       g_free(ts);
     }
 }
@@ -695,6 +726,7 @@ on_edit_save_clicked (GtkButton * button, gpointer user_data)
   GtkWidget *edit = (GtkWidget *) user_data;
   GSList *tags;
   struct person *p = g_object_get_data (G_OBJECT (edit), "person");
+  gchar *nametext = NULL;
   
   for (tags = g_object_get_data (G_OBJECT (edit), "tag-widgets");
        tags; tags = tags->next)
@@ -735,10 +767,63 @@ on_edit_save_clicked (GtkButton * button, gpointer user_data)
         }
       tag = g_object_get_data (G_OBJECT (w), "db-tag");
       db_set_data (p, tag, text);
+      
+      /* remember name */
+      if (!strcasecmp(tag, "NAME"))
+        {
+            nametext = text;
+        }
     }
 
   retrieve_special_fields (edit, p);
 
+    /* handle name details */
+    if (nametext)
+      {
+          char e1[101];
+          char e2[101];
+          char e3[101];
+          char e4[101];
+          int num_elements = sscanf(nametext, "%100s %100s %100s %100s", e1, e2, e3, e4);
+          if (num_elements == 2)
+            {
+              if (strstr(nametext, ","))
+                {
+                  if (strstr(e1,",")) 
+                    strstr(e1,",")[0] = 0;
+                  db_set_data(p, "GIVEN_NAME", g_strdup(e2));
+                  db_set_data(p, "FAMILY_NAME", g_strdup(e1));
+                }
+              else
+                {
+                  db_set_data(p, "GIVEN_NAME", g_strdup(e1));
+                  db_set_data(p, "FAMILY_NAME", g_strdup(e2));
+                }
+            }
+              
+          if (num_elements > 2)
+            {
+              db_set_data(p, "TITLE", g_strdup(e1));
+              if (strstr(nametext, ","))
+                {
+                  if (strstr(e2,",")) 
+                    strstr(e2,",")[0] = 0;
+                  db_set_data(p, "GIVEN_NAME", g_strdup(e3));
+                  db_set_data(p, "FAMILY_NAME", g_strdup(e2));
+                }
+              else
+                {
+                  db_set_data(p, "GIVEN_NAME", g_strdup(e2));
+                  db_set_data(p, "FAMILY_NAME", g_strdup(e3));
+                }
+            }
+          if (num_elements > 3)
+            {
+              db_set_data(p, "HONORIFIC_SUFFIX", g_strdup(e4));
+            }
+      }
+  
+  
   if (commit_person (p))
     {
       gtk_widget_destroy (edit);
