@@ -8,7 +8,7 @@
  */
 
 /*
- * $Id: main.c,v 1.71 2004/07/14 18:01:41 florian Exp $
+ * $Id: main.c,v 1.80 2004/08/10 16:52:11 florian Exp $
  *
  */
 
@@ -38,6 +38,7 @@
 #include "proto.h"
 #include "main.h"
 #include "export.h"
+#include "import-vcard.h"
 
 #define SYSTEM_PIXMAPS_DIR PREFIX "/share/pixmaps/"
 #define MY_PIXMAPS_DIR "gpe-contacts/"
@@ -203,7 +204,7 @@ get_tag_name_1(const gchar *tag, edit_thing_t e)
       case ITEM_IMAGE:
       case ITEM_SINGLE_LINE:
       case ITEM_MULTI_LINE:
-        if (!strcmp(tag,e->tag))
+        if (!strcasecmp(tag,e->tag))
           return e->name;
       break;
     } 
@@ -257,8 +258,10 @@ show_details (struct person *p)
           {
             struct tag_value *id = (struct tag_value *) iter->data;
             fieldname = get_tag_name(id->tag);
-            if (fieldname)
+            if ((fieldname) && strcmp(id->tag,"NAME"))
               {
+                if (fieldname[strlen(fieldname)-1] == ':')
+                  fieldname[strlen(fieldname)-1] = 0;
                 if (strstr(id->tag,"HOME"))
                   fielddesc = g_strdup_printf("%s (%s):",fieldname,_("home"));
                 else if (strstr(id->tag,"WORK"))
@@ -280,6 +283,8 @@ show_details (struct person *p)
                 gchar *ts;
                 if (!strcmp(id->tag,"NAME")) 
                  {
+                    if (fieldname[strlen(fieldname)-1] == ':')
+                      fieldname[strlen(fieldname)-1] = 0;
                     lleft = gtk_label_new(NULL);
                     ts = g_strdup_printf("<b>%s:</b>",_("Name"));
                     gtk_label_set_markup(GTK_LABEL(lleft),ts);
@@ -395,37 +400,74 @@ selection_made (GtkTreeSelection *sel, GObject *o)
 static gboolean
 match_for_search (struct person *p, const gchar *text, struct gpe_pim_category *cat)
 {
-  gchar *lname = g_utf8_strdown (p->name, -1);
-
-  if (strstr (lname, text) == NULL)
+  gchar *gn;
+  gchar *fn;
+  gchar *name;
+  
+  if (p->given_name)
+    gn = g_utf8_strdown(p->given_name, -1);
+  else
+    gn = g_strdup("");
+  
+  if (p->family_name)
+    fn = g_utf8_strdown(p->family_name, -1);
+  else
+    fn = g_strdup("");
+  
+  if ((p->name) && strlen(p->name))
+    name = g_utf8_strdown(p->name, -1);
+  else
+    name = g_strdup("");
+  
+  if (!g_str_has_prefix(gn,text) && !g_str_has_prefix(fn,text) 
+	  && !g_str_has_prefix(name,text))
     {
-      g_free (lname);
-      return FALSE;
+      if (gn) g_free(gn);
+      if (fn) g_free(fn);
+      if (fn) g_free(name);
+        return FALSE;
+    }
+  if (gn) g_free(gn);
+  if (fn) g_free(fn);
+  if (fn) g_free(name);
+    
+/* alternate method if nothing found 
+  if ...
+    
+  else
+    {
+      gchar *lname = g_utf8_strdown (p->name, -1);
+    
+      if (strstr (lname, text) == NULL)
+        {
+          g_free (lname);
+          return FALSE;
+        }
     }
 
   g_free (lname);
-
+*/
   if (cat)
     {
       GSList *l;
       gboolean found = FALSE;
 
       for (l = p->data; l; l = l->next)
-	{
-	  struct tag_value *v = l->data;
-	  if (!strcasecmp (v->tag, "CATEGORY") && v->value)
-	    {
-	      guint c = atoi (v->value);
-	      if (c == cat->id)
-		{
-		  found = TRUE;
-		  break;
-		}
-	    }
-	}
+        {
+          struct tag_value *v = l->data;
+          if (!strcasecmp (v->tag, "CATEGORY") && v->value)
+            {
+              guint c = atoi (v->value);
+              if (c == cat->id)
+                {
+                  found = TRUE;
+                  break;
+                }
+            }
+        }
 
       if (!found)
-	return FALSE;
+        return FALSE;
     }
 
   return TRUE;
@@ -445,7 +487,7 @@ do_search (GObject *obj, GtkWidget *entry)
       GSList *ll = g_slist_nth (l, category - 1);
 
       if (ll)
-	c = ll->data;
+        c = ll->data;
 
       g_slist_free (l);
     }
@@ -460,10 +502,10 @@ do_search (GObject *obj, GtkWidget *entry)
       GtkTreeIter iter;
 
       if (match_for_search (p, text, c))
-	{
-	  gtk_list_store_append (list_store, &iter);
-	  gtk_list_store_set (list_store, &iter, 0, p->name, 1, p->id, -1);
-	}
+        {
+	      gtk_list_store_append (list_store, &iter);
+	      gtk_list_store_set (list_store, &iter, 0, p->name, 1, p->id, -1);
+        }
 
       discard_person (p);
     }
@@ -519,28 +561,32 @@ window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkTreeView *tree)
   GtkTreePath *path;
   
   if (k->keyval == GDK_Up)
-  {
-    gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree),&path,NULL);
-    if (path) 
-      gtk_tree_path_prev(path);
-    else
-      path = gtk_tree_path_new_first();
-    gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree),path,NULL,FALSE);
-    gtk_tree_path_free(path);
-    return TRUE;
-  }
+    {
+      gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree),&path,NULL);
+      if (path) 
+        gtk_tree_path_prev(path);
+      else
+        path = gtk_tree_path_new_first();
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree),path,NULL,FALSE);
+      gtk_tree_path_free(path);
+      return TRUE;
+    }
   if (k->keyval == GDK_Down)
-  {
-    gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree),&path,NULL);
-    if (path)
-      gtk_tree_path_next(path);
-    else
-      path = gtk_tree_path_new_first();
-    gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree),path,NULL,FALSE);
-    gtk_tree_path_free(path);
-    return TRUE;
-  }
-  
+    {
+      gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree),&path,NULL);
+      if (path)
+        gtk_tree_path_next(path);
+      else
+        path = gtk_tree_path_new_first();
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree),path,NULL,FALSE);
+      gtk_tree_path_free(path);
+      return TRUE;
+    }
+
+  if (k->keyval == GDK_Return)
+    {
+      edit_contact(NULL, NULL);
+    }  
   return FALSE;
 }
 
@@ -548,7 +594,7 @@ static gboolean
 toolbar_key_press_event (GtkWidget *widget, GdkEventKey *k)
 {
   
-  if (k->keyval == GDK_Tab)
+  if ((k->keyval == GDK_Tab) || (k->keyval == GDK_Up))
   {
     gtk_widget_grab_focus(search_entry);
     return TRUE;
@@ -556,6 +602,22 @@ toolbar_key_press_event (GtkWidget *widget, GdkEventKey *k)
   return FALSE;
 }
 
+static gboolean
+search_entry_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *toolitem)
+{
+  if (k->keyval == GDK_Down)
+  {
+    gtk_widget_grab_focus(toolitem);
+    return TRUE;
+  }
+  if (k->keyval == GDK_Escape)
+  {
+    gtk_editable_delete_text(GTK_EDITABLE(widget),0,-1);
+    return TRUE;
+  }
+  return FALSE;
+}
+
 static gboolean
 list_button_press_event (GtkWidget *widget, GdkEventButton *b, GtkListStore *list_store)
 {
@@ -649,19 +711,24 @@ create_main (gboolean show_config_button)
   GtkWidget *pDetail;
   GtkWidget *tabDetail;
   GtkWidget *toolbar, *pw;
-  GtkWidget *b;
+  GtkWidget *b, *btnNew;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkTreeSelection *tree_sel;
   GtkWidget *scrolled_window;
   gint size_x, size_y;
 
-  size_x = gdk_screen_width() / 4;
-  size_y = gdk_screen_height() / 3;  
-  if (size_x < 240) size_x = 240;
-  if (size_y < 320) size_y = 320;
+  /* screen layout detection */
+  size_x = gdk_screen_width();
+  size_y = gdk_screen_height();  
   mode_large_screen = (size_x > 240) && (size_y > 320);
   mode_landscape = (size_x > size_y);
+  size_x /= 4;
+  size_y /= 3;
+  if (size_x < 240) size_x = 240;
+  if (size_y < 320) size_y = 320;
+
+  /* main window */
   main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (main_window), _("Contacts"));
   gtk_window_set_default_size (GTK_WINDOW (main_window), size_x, size_y);
@@ -675,7 +742,7 @@ create_main (gboolean show_config_button)
 
   gtk_box_pack_start (GTK_BOX (vbox1), toolbar, FALSE, FALSE, 0);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW,
+  btnNew = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW,
 			    _("New contact"), _("Tap here to add a new contact."),
 			    G_CALLBACK (new_contact), NULL, -1);
 
@@ -770,6 +837,8 @@ create_main (gboolean show_config_button)
 
   g_signal_connect (G_OBJECT (toolbar), "key_press_event", 
 		    G_CALLBACK (toolbar_key_press_event), NULL);
+  g_signal_connect (G_OBJECT (entry1), "key_press_event", 
+		    G_CALLBACK (search_entry_key_press_event), btnNew);
             
   search_entry = entry1;
 
@@ -802,11 +871,32 @@ create_main (gboolean show_config_button)
   return main_window;
 }
 
+
+static int 
+import_one_file(const gchar *filename)
+{
+  GError *err = NULL;
+  gchar *content = NULL;
+  gsize count = 0;
+  int result = 0;
+	
+  if (g_file_get_contents(filename, &content, &count, &err))
+    {
+      result = import_vcard(content, count);
+      g_free(content);
+    }
+  else
+    result = -1;
+  return result;
+}
+
+
 int
 main (int argc, char *argv[])
 {
   int arg;
   gboolean show_config_button = TRUE;
+  gchar *ifile = NULL;
   
 #ifdef ENABLE_NLS
   setlocale (LC_ALL, "");
@@ -815,7 +905,7 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
 #endif
-	
+ 	
   if (gpe_application_init (&argc, &argv) == FALSE)
     exit (1);
 
@@ -823,21 +913,49 @@ main (int argc, char *argv[])
     exit (1);
 
   /* check command line args */
-  while ((arg = getopt(argc, argv, "n")) >= 0)
+  while ((arg = getopt(argc, argv, "ni:")) >= 0)
   {
-    if (arg == 'n') 
+    if (arg == 'n')
       {
         show_config_button = FALSE;
         break;
       }
+    if (arg == 'i')
+		ifile = optarg;
   }    
-  
-  if (db_open ())
-    exit (1);
 
+  /* are we called to import a file? */
+  if (ifile)
+    {
+      int ret;
+      /*GtkWidget *dialog = NULL;*/
+      
+      ret =  import_one_file(ifile);
+      if (ret)
+        printf("Could not import file %s.\n", ifile);
+      else
+        printf("Successfully imported %s.\n", ifile);
+/*       dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
+	                                    GTK_BUTTONS_OK,
+	                                    _("Could not import file %s."),
+	                                     ifile);
+	  else
+        dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
+	                                      GTK_BUTTONS_OK,
+	                                      _("File %s imported sucessfully."),
+	                                      ifile);
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(dialog);
+*/      
+      exit (EXIT_SUCCESS);
+    }
+  
   export_init ();
 
   displaymigration_init ();
+  
+  if (db_open ())
+    exit (1);
 
   load_well_known_tags ();
 
